using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Net;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Windows.Forms;
using Microsoft.WindowsMobile.PocketOutlook;

namespace GMailNotifier
{
	public partial class Form1 : Form
	{
		private Thread m_update = null;
		private static readonly string m_appointmentBodyPrefix = "Check your gmail messages";
		private readonly string m_appointmentBody = String.Format("{0} at {1}", m_appointmentBodyPrefix, GmailUrl);
		private enum MailState
		{
			Broken,
			NoMail,
			NewMail
		};

		public Form1()
		{
			InitializeComponent();
		}

		#region Thread-Safe functions
		private delegate void SetTextCallback(string text);
		private void SetText(string text)
		{
			if (label1.InvokeRequired)
			{
				this.Invoke(new SetTextCallback(SetText), new object[] { text });
			}
			else
			{
				if (String.IsNullOrEmpty(text))
					label3.Text = String.Empty;
				else
					label3.Text = text;
				Application.DoEvents();
			}
		}

		private delegate void SetUpdateCallback();
		private void SetUpdate()
		{
			if (label1.InvokeRequired)
			{
				this.Invoke(new SetUpdateCallback(SetUpdate));
			}
			else
			{
				label2.Text = String.Format("Last Check: {0}", DateTime.Now.ToShortTimeString());
				Application.DoEvents();
			}
		}

		private delegate void EnableUICallback(bool enabled);
		private void EnableUI(bool enabled)
		{
			if (this.InvokeRequired)
			{
				this.Invoke(new EnableUICallback(EnableUI), new object[] { enabled });
			}
			else
			{
				menuItem1.Enabled = enabled;
				menuItem4.Enabled = enabled;
				Application.DoEvents();
			}
		}

		private delegate void SetIconCallback(MailState state);
		private void SetIcon(MailState state)
		{
			if (this.InvokeRequired)
			{
				this.Invoke(new SetIconCallback(SetIcon), new object[] { state });
			}
			else
			{
				switch (state)
				{
					case MailState.NoMail:
						this.Icon = Properties.Resources.gnotify_4;
						break;
					case MailState.NewMail:
						this.Icon = Properties.Resources.gnotify_2;
						break;
					case MailState.Broken:
					default:
						this.Icon = Properties.Resources.gnotify_5;
						break;
				}
				Application.DoEvents();
			}
		}
		#endregion

		private void UpdateAppointment(string title, string inbox)
		{
			lock (this)
			{
				bool found = false;
				using (OutlookSession outlooksession = new OutlookSession())
				{
					for (int i = 0; i < outlooksession.Appointments.Items.Count; i++)
					{
						Appointment app = outlooksession.Appointments.Items[i];
						if (app.AllDayEvent && app.IsRecurring && app.Body.StartsWith(m_appointmentBodyPrefix))
						{
							app.Subject = title;
							app.Body = m_appointmentBody + "\r\n\r\n" + inbox;
							found = true;
							app.Update();
							break;
						}
					}

					if (found == false)
					{
						Appointment app = new Appointment();
						app.AllDayEvent = true;
						app.ReminderSet = false;
						app.BusyStatus = BusyStatus.Free;
						app.Start = DateTime.Today;
						app.Duration = new TimeSpan(1, 0, 0, 0);
						app.Body = m_appointmentBody + "\r\n\r\n" + inbox;
						app.RecurrencePattern.RecurrenceType = RecurrenceType.Daily;
						app.Start = DateTime.Today;
						app.Subject = title;
						outlooksession.Appointments.Items.Add(app);
					}
				}
			}
		}

		private void DeleteAppointment()
		{
			lock (this)
			{
				using (OutlookSession outlooksession = new OutlookSession())
				{
					for (int i = 0; i < outlooksession.Appointments.Items.Count; i++)
					{
						Appointment app = outlooksession.Appointments.Items[i];
						if (app.AllDayEvent && app.IsRecurring && app.Body.StartsWith(m_appointmentBodyPrefix))
						{
							app.Delete();
							break;
						}
					}
				}
			}
		}

		private static string GmailUrl
		{
			get
			{
				string url = "https://mail.google.com/";
				if (!Settings.IsGmail)
					url += String.Format("a/{0}/", Settings.Domain);
				else
					url += "mail/";
				return (url);
			}
		}


		private void Form1_Load(object sender, EventArgs e)
		{
			label1.Text = String.Format("User: {0}", Settings.User);
			SetIcon(MailState.NoMail);
			timer1.Enabled = false;
			timer1.Interval = (Settings.IntervalSeconds * 1000);
			timer1.Enabled = true;
			menuItem2_Click(null, null);
		}

		private void GmailThread()
		{
			bool success = false;
			EnableUI(false);

			try
			{
				SetText("Checking mail...");
				WebRequest wr = WebRequest.Create(new Uri(String.Format("{0}feed/atom", GmailUrl)));
				wr.Credentials = new NetworkCredential(Settings.User, Settings.Password);
				using (WebResponse response = wr.GetResponse())
				{
					using (Stream s = response.GetResponseStream())
					{
						using (StreamReader sr = new StreamReader(s))
						{
							string r = sr.ReadToEnd();
							Match m = Regex.Match(r, "<fullcount>([^<]*)");
							if (m.Success)
							{
								string inbox = "";
								string mailCount = m.Groups[1].Value;
								SetText(String.Format("New Messages: {0}", mailCount));
								success = true;

								int idx = r.IndexOf("<entry>");
								string rest = null;
								if (idx != -1)
									rest = r.Substring(idx);
								while (rest != null)
								{
									string entry = rest;
									idx = entry.IndexOf("</entry>");
									if (idx != -1)
									{
										rest = entry.Substring(idx + 8);
										entry = entry.Substring(0, idx);
									}
									else
										rest = null;

									if (entry.StartsWith("<entry>"))
									{
										string subject = "";
										string content = "";
										string author = "";
										string email = "";

										entry = entry.Substring(7);
										m = Regex.Match(entry, "<title>([^<]*)");
										if (m.Success)
											subject = m.Groups[1].Value;
										m = Regex.Match(entry, "<summary>([^<]*)");
										if (m.Success)
											content = m.Groups[1].Value;
										m = Regex.Match(entry, "<name>([^<]*)");
										if (m.Success)
											author = m.Groups[1].Value;
										m = Regex.Match(entry, "<email>([^<]*)");
										if (m.Success)
											email = m.Groups[1].Value;
										content = content.Replace("&amp;", "&");
										content = System.Web.HttpUtility.HtmlDecode(content);
										string message = String.Format("From: {0} ({1})", email, author) + "\r\n" +
											String.Format("Subject: {0}", subject) + "\r\n" +
											content + "\r\n\r\n";
										inbox += message;
									}
								}

								if (mailCount == "0")
								{
									SetIcon(MailState.NoMail);
									DeleteAppointment();
								}
								else
								{
									SetIcon(MailState.NewMail);
									UpdateAppointment(String.Format("{0} ({1})", Settings.Email, mailCount), inbox);
								}
							}
						}
					}
				}

				if(!success)
					SetText("Error: Couldn't log into server, check your settings");
			}
			catch (Exception ex)
			{
				SetText(String.Format("Error: {0}", ex.Message));
				success = false;
			}

			if (!success)
				SetIcon(MailState.Broken);

			SetUpdate();

			EnableUI(true);
			m_update = null;
		}

		private void menuItem5_Click_1(object sender, EventArgs e)
		{
			Settings s = new Settings();
			if (s.ShowDialog() == DialogResult.OK)
			{
				Form1_Load(null, null);
			}
		}

		private void menuItem2_Click(object sender, EventArgs e)
		{
			lock (this)
			{
				if (!String.IsNullOrEmpty(Settings.User) && m_update == null)
				{
					m_update = new Thread(new ThreadStart(GmailThread));
					m_update.Start();
				}
			}
		}

		private void menuItem3_Click(object sender, EventArgs e)
		{
			System.Diagnostics.Process.Start(GmailUrl, "");
		}

		private void menuItem6_Click(object sender, EventArgs e)
		{
			DeleteAppointment();
			Application.Exit();
		}

		private void timer1_Tick(object sender, EventArgs e)
		{
			menuItem2_Click(null, null);
		}
	}
}